@@ -65,8 +65,12 @@ from accern_xyme.types import (
     JobInfo,
     JobList,
     JSONBlobResponse,
+    KafkaGroup,
     KafkaMessage,
+    KafkaOffsets,
+    KafkaThroughput,
     KafkaTopics,
+    ListNamedSecretKeys,
     MaintenanceResponse,
     MinimalQueueStatsResponse,
     ModelParamsResponse,
@@ -90,6 +94,7 @@ from accern_xyme.types import (
     QueueStatsResponse,
     QueueStatus,
     ReadNode,
+    SetNamedSecret,
     TaskStatus,
     Timing,
     TimingResult,
@@ -106,7 +111,7 @@ else:
     WVD = weakref.WeakValueDictionary
 
 
-__version__ = "0.1.10"
+__version__ = "0.1.11"
 # FIXME: async calls, documentation, auth, summary – time it took etc.
 
 
@@ -475,6 +480,7 @@ class XYMEClient:
                     res = quick_server.worker_request(url, args)
                     if "errMessage" in res:
                         raise ServerSideError(res["errMessage"])
+                    return res
                 except quick_server.WorkerError as e:
                     if e.get_status_code() == 403:
                         raise AccessDenied(e.args) from e
@@ -737,6 +743,10 @@ class XYMEClient:
                 "reset": int(reset),
             }))
 
+    def reset_cache(self) -> CacheStats:
+        return cast(CacheStats, self._request_json(
+            METHOD_POST, "/cache_reset", {}))
+
     def create_kafka_error_topic(self) -> KafkaTopics:
         return cast(KafkaTopics, self._request_json(
             METHOD_POST, "/kafka_topics", {
@@ -753,6 +763,17 @@ class XYMEClient:
         return cast(List[str], self._request_json(
             METHOD_GET, "/kafka_msg", {}))
 
+    def get_named_secret_keys(self) -> List[str]:
+        return cast(ListNamedSecretKeys, self._request_json(
+            METHOD_GET, "/named_secrets", {}))["keys"]
+
+    def set_named_secret(self, key: str, value: str) -> bool:
+        return cast(SetNamedSecret, self._request_json(
+            METHOD_PUT, "/named_secrets", {
+                "key": key,
+                "value": value,
+            }))["replaced"]
+
 
 # *** XYMEClient ***
 
@@ -1352,7 +1373,7 @@ class PipelineHandle:
         return [msgs[key] for key in names]
 
     def read_kafka_output(
-            self, single: bool = False) -> Optional[ByteResponse]:
+            self, max_rows: int = 100) -> Optional[ByteResponse]:
 
         def read_single() -> Tuple[ByteResponse, str]:
             cur, read_ctype = self._client.request_bytes(
@@ -1361,7 +1382,7 @@ class PipelineHandle:
                 })
             return interpret_ctype(cur, read_ctype), read_ctype
 
-        if single:
+        if max_rows <= 1:
             return read_single()[0]
 
         res: List[ByteResponse] = []
@@ -1376,10 +1397,109 @@ class PipelineHandle:
                 raise ValueError(
                     f"inconsistent return types {ctype} != {cur_ctype}")
             res.append(val)
+            if len(res) >= max_rows:
+                break
         if not res or ctype is None:
             return None
         return merge_ctype(res, ctype)
 
+    def get_kafka_offsets(self, alive: bool) -> KafkaOffsets:
+        return cast(KafkaOffsets, self._client._request_json(
+            METHOD_GET, "/kafka_offsets", {
+                "pipeline": self._pipe_id,
+                "alive": int(alive),
+            }))
+
+    def get_kafka_throughput(
+            self,
+            segment_interval: float = 120.0,
+            segments: int = 5) -> KafkaThroughput:
+        assert segments > 0
+        assert segment_interval > 0.0
+        offsets = self.get_kafka_offsets(alive=False)
+        now = time.monotonic()
+        measurements: List[Tuple[int, int, int, float]] = [(
+            offsets["input"],
+            offsets["output"],
+            offsets["error"],
+            now,
+        )]
+        for _ in range(segments):
+            prev = now
+            while now - prev < segment_interval:
+                time.sleep(max(0.0, segment_interval - (now - prev)))
+                now = time.monotonic()
+            offsets = self.get_kafka_offsets(alive=False)
+            measurements.append((
+                offsets["input"],
+                offsets["output"],
+                offsets["error"],
+                now,
+            ))
+        first = measurements[0]
+        last = measurements[-1]
+        total_input = last[0] - first[0]
+        total_output = last[1] - first[1]
+        errors = last[2] - first[2]
+        total = last[3] - first[3]
+        input_segments: List[float] = []
+        output_segments: List[float] = []
+        cur_input = first[0]
+        cur_output = first[1]
+        cur_time = first[3]
+        for (next_input, next_output, _, next_time) in measurements[1:]:
+            seg_time = next_time - cur_time
+            input_segments.append((next_input - cur_input) / seg_time)
+            output_segments.append((next_output - cur_output) / seg_time)
+            cur_input = next_input
+            cur_output = next_output
+            cur_time = next_time
+        inputs = pd.Series(input_segments)
+        outputs = pd.Series(output_segments)
+        return {
+            "pipeline": self._pipe_id,
+            "input": {
+                "throughput": total_input / total,
+                "max": inputs.max(),
+                "min": inputs.min(),
+                "stddev": inputs.std(),
+                "segments": segments,
+                "count": total_input,
+                "total": total,
+            },
+            "output": {
+                "throughput": total_output / total,
+                "max": outputs.max(),
+                "min": outputs.min(),
+                "stddev": outputs.std(),
+                "segments": segments,
+                "count": total_output,
+                "total": total,
+            },
+            "faster": "both" if total_input == total_output else (
+                "input" if total_input > total_output else "output"),
+            "errors": errors,
+        }
+
+    def get_kafka_group(self) -> KafkaGroup:
+        return cast(KafkaGroup, self._client._request_json(
+            METHOD_GET, "/kafka_group", {
+                "pipeline": self._pipe_id,
+            }))
+
+    def set_kafka_group(
+            self,
+            group_id: Optional[str] = None,
+            reset: Optional[str] = None,
+            **kwargs: Any) -> KafkaGroup:
+        return cast(KafkaGroup, self._client._request_json(
+            METHOD_PUT, "/kafka_group", {
+                "pipeline": self._pipe_id,
+                "group_id": group_id,
+                "reset": reset,
+                **kwargs,
+            }))
+
     def __hash__(self) -> int:
         return hash(self._pipe_id)
 
@@ -1533,7 +1653,7 @@ class NodeHandle:
             "error": "!",
             "unknown": "?",
             "virtual": "∴" if allow_unicode else "V",
-            "parallel": "=",
+            "queue": "=",
         }
         return status_map[self.get_status()]
 
