@@ -2,8 +2,8 @@ from typing import List, Set
 import uuid
 from typing_extensions import Literal
 
-from .accern_xyme import PipelineHandle, NodeHandle
-from .types import PipelineDef, NodeDef
+from accern_xyme.accern_xyme import PipelineHandle, NodeHandle
+from accern_xyme.types import PipelineDef, NodeDef
 
 
 ColType = Literal[  # pylint: disable=invalid-name
@@ -169,7 +169,7 @@ class DataPipeline:
             "nodes": node_defs,
             "state": pipe.get_state_type(),
             "high_priority": pipe.is_high_priority(),
-            "is_parallel": pipe.is_parallel(),
+            "queue_mng": pipe.get_queue_mng(),
         }
         pipe.set_pipeline(pipe_def)
         self._dirty = False
