@@ -13,7 +13,7 @@ TaskStatus = Literal[
     "error",
     "unknown",
     "virtual",
-    "parallel",
+    "queue",
 ]
 ParamType = Literal[
     "str",
@@ -218,6 +218,12 @@ FlushAllQueuesResponse = TypedDict('FlushAllQueuesResponse', {
 WorkerScale = TypedDict('WorkerScale', {
     "success": bool,
 })
+SetNamedSecret = TypedDict('SetNamedSecret', {
+    "replaced": bool,
+})
+ListNamedSecretKeys = TypedDict('ListNamedSecretKeys', {
+    "keys": List[str],
+})
 KafkaTopics = TypedDict('KafkaTopics', {
     "topics": Dict[str, Optional[str]],
     "create": bool,
@@ -225,6 +231,32 @@ KafkaTopics = TypedDict('KafkaTopics', {
 KafkaMessage = TypedDict('KafkaMessage', {
     "messages": Dict[str, str],
 })
+KafkaOffsets = TypedDict('KafkaOffsets', {
+    "error": int,
+    "input": int,
+    "output": int,
+})
+KafkaGroup = TypedDict('KafkaGroup', {
+    "group": str,
+    "pipeline": str,
+    "reset": Optional[str],
+})
+ThroughputDict = TypedDict('ThroughputDict', {
+    "throughput": float,
+    "max": float,
+    "min": float,
+    "stddev": float,
+    "segments": int,
+    "count": int,
+    "total": float,
+})
+KafkaThroughput = TypedDict('KafkaThroughput', {
+    "pipeline": str,
+    "input": ThroughputDict,
+    "output": ThroughputDict,
+    "faster": Literal["input", "output", "both"],
+    "errors": int,
+})
 PutNodeBlob = TypedDict('PutNodeBlob', {
     "key": str,
     "new_uri": str,
